
from datetime import date
from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel

class LoginSuccess(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    is_pass_code_enabled: bool | None = Field(default=None)
    locale: str | None = Field(default=None)
    redirect_url: str | None = Field(default=None)
    session_id: str
    status: int | None = Field(default=None)
    status_text: str | None = Field(default=None)

class LoginError(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    login_failures: int| None = Field(default=None)
    error: str | None = Field(default=None)
    path: str | None = Field(default=None)
    status: int
    status_text: str | None = Field(default=None)
    timestamp: date | None = Field(default=None)


class Login(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    username: str
    password: str
    is_pass_code_reset: bool = Field(default=False)
    is_redirect_to_mobile: bool = Field(default=False)
    query_tarams: dict = Field(default_factory=dict)
    one_time_password: str | None = Field(default=None)



from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class ProductRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )


class ProductsConfig(ProductRequest):
    values: dict


class BondsRequest(ProductRequest):
    bond_issuer_type_id: int
    bond_exchange_id: int
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class ETFsRequest(ProductRequest):
    popular_only: bool
    input_aggregate_types: str
    input_aggregate_values: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class FundsRequest(ProductRequest):
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class FuturesRequest(ProductRequest):
    future_exchange_id: int
    underlying_isin: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class LeveragedsRequest(ProductRequest):
    popular_only: bool
    input_aggregate_types: str
    input_aggregate_values: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class LookupRequest(ProductRequest):
    search_text: str
    limit: int
    offset: int
    product_type_id: int


class OptionsRequest(ProductRequest):
    input_aggregate_types: str
    input_aggregate_values: str
    option_exchange_id: int
    underlying_isin: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class StocksRequest(ProductRequest):
    exchange_id: int | None = Field(default=None)
    is_in_us_green_list: bool | None = Field(default=None, alias="isInUSGreenList")
    index_id: int | None = Field(default=None)
    offset: int = Field(default=10)
    limit: int = Field(default=0)
    require_total: bool = Field(default=False)
    search_text: str | None = Field(default=None)
    sort_columns: str
    sort_types: str
    stock_country_id: int


class WarrantsRequest(ProductRequest):
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class ProductBatch(BaseModel):
    offset: int
    products: list[dict]
    response_datetime: datetime = Field(default_factory=datetime.now)
    total: int = Field(default=0)



# QUOTECAST RELATED
CHART = "https://charting.vwdservices.com/hchart/v1/deGiro/data.js?"
QUOTECAST = "https://degiro.quotecast.vwdservices.com/CORS"
QUOTECAST_VERSION = "1.0.20201211"

# TRADING RELATED
ACCOUNT_INFO = "https://trader.degiro.nl/trading/secure/v5/account/info"
ACCOUNT_OVERVIEW = "https://trader.degiro.nl/reporting/secure/v6/accountoverview"
AGENDA = "https://trader.degiro.nl/dgtbxdsservice/agenda/v2"
CASH_ACCOUNT_REPORT = "https://trader.degiro.nl/reporting/secure/v3/cashAccountReport"
CLIENT_DETAILS = "https://trader.degiro.nl/pa/secure/client"
COMPANY_PROFILE = "https://trader.degiro.nl/dgtbxdsservice/company-profile/v2"
COMPANY_RATIOS = "https://trader.degiro.nl/dgtbxdsservice/company-ratios"
CONFIG = "https://trader.degiro.nl/login/secure/config"
ESTIMATES_SUMMARIES = "https://trader.degiro.nl/dgtbxdsservice/estimates-summaries"
FINANCIAL_STATEMENTS = "https://trader.degiro.nl/dgtbxdsservice/financial-statements"
LATEST_NEWS = "https://trader.degiro.nl/dgtbxdsservice/newsfeed/v2/latest-news"
LOGIN = "https://trader.degiro.nl/login/secure/login"
LOGOUT = "https://trader.degiro.nl/trading/secure/logout"
NEWS_BY_COMPANY = "https://trader.degiro.nl/dgtbxdsservice/newsfeed/v2/news-by-company"
ORDER_CHECK = "https://trader.degiro.nl/trading/secure/v5/checkOrder"
ORDER_CONFIRM = "https://trader.degiro.nl/trading/secure/v5/order"
ORDER_DELETE = "https://trader.degiro.nl/trading/secure/v5/order"
ORDERS_HISTORY = "https://trader.degiro.nl/reporting/secure/v4/order-history"
ORDER_UPDATE = "https://trader.degiro.nl/trading/secure/v5/order"
POSITION_REPORT = "https://trader.degiro.nl/reporting/secure/v3/positionReport"
FAVOURITES_LIST = "https://trader.degiro.nl/favorites/secure/v1"
PRODUCT_SEARCH_BONDS = "https://trader.degiro.nl/product_search/secure/v5/bonds"
PRODUCT_SEARCH_ETFS = "https://trader.degiro.nl/product_search/secure/v5/etfs"
PRODUCT_SEARCH_FUNDS = "https://trader.degiro.nl/product_search/secure/v5/funds"
PRODUCT_SEARCH_FUTURES = "https://trader.degiro.nl/product_search/secure/v5/futures"
PRODUCT_SEARCH_LEVERAGEDS = (
    "https://trader.degiro.nl/product_search/secure/v5/leverageds"
)
PRODUCT_SEARCH_LOOKUP = (
    "https://trader.degiro.nl/product_search/secure/v5/products/lookup"
)
PRODUCT_SEARCH_OPTIONS = "https://trader.degiro.nl/product_search/secure/v5/options"
PRODUCT_SEARCH_STOCKS = "https://trader.degiro.nl/product_search/secure/v5/stocks"
PRODUCT_SEARCH_WARRANTS = "https://trader.degiro.nl/product_search/secure/v5/warrants"
PRODUCTS_INFO = "https://trader.degiro.nl/product_search/secure/v5/products/info"
PRODUCTS_CONFIG = "https://trader.degiro.nl/product_search/config/dictionary"
TOP_NEWS_PREVIEW = (
    "https://trader.degiro.nl/dgtbxdsservice/newsfeed/v2/top-news-preview"
)
TRANSACTIONS_HISTORY = "https://trader.degiro.nl/reporting/secure/v4/transactions"
UPDATE = "https://trader.degiro.nl/trading/secure/v5/update"



from pydantic import BaseModel, Field, validator


class Credentials(BaseModel):
    int_account: int | None = Field(default=None)
    username: str
    password: str
    totp_secret_key: str | None = Field(default=None)
    one_time_password: int | None = Field(default=None)

    @validator("totp_secret_key")
    @classmethod
    def one_of(cls, v, values, **kwargs):
        _ = kwargs
        if "one_time_password" in values and values["one_time_password"] is not None:
            raise ValueError(
                "You can't set both `one_time_password` and `totp_secret_key`."
            )
        return v



import logging
from typing import Union

import requests
from google.protobuf import json_format

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.trading_pb2 import (
    Favourites,
)


class ActionGetFavouritesList(AbstractAction):
    @staticmethod
    def favourites_to_grpc(payload: dict) -> Favourites:
        favourites = Favourites()
        favourites.response_datetime.GetCurrentTime()
        json_format.ParseDict(
            js_dict={"values": payload["data"]},
            message=favourites,
            ignore_unknown_fields=False,
            descriptor_pool=None,
        )

        return favourites

    @classmethod
    def get_favourites_list(
        cls,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> Union[Favourites, dict, None]:
        """Retrieve the lists of favourite products.
        Args:
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            Favourites: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.FAVOURITES_LIST

        params = {
            "intAccount": int_account,
            "sessionId": session_id,
        }

        request = requests.Request(method="GET", url=url, params=params)
        prepped = session.prepare_request(request)
        response_raw = None

        try:
            response_raw = session.send(prepped)
            response_raw.raise_for_status()
            response_dict = response_raw.json()

            if raw is True:
                return response_dict
            else:
                return cls.favourites_to_grpc(
                    payload=response_dict,
                )
        except requests.HTTPError as e:
            status_code = getattr(response_raw, "status_code", "No status_code found.")
            text = getattr(response_raw, "text", "No text found.")
            logger.fatal(status_code)
            logger.fatal(text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        raw: bool = False,
    ) -> Union[Favourites, dict, None]:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_favourites_list(
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



from datetime import datetime, timedelta
from typing import Literal

from pydantic import BaseModel, Field

from degiro_connector.quotecast.models.metric import MetricType


class Ticker(BaseModel):
    """Text JSON message received from Degiro's Quotecast API."""

    json_text: str
    response_datetime: datetime = Field(default_factory=datetime.now)
    request_duration: timedelta


class TickerRequest(BaseModel):
    request_type: Literal["subscription", "unsubscription"]
    request_map: dict[str, list[MetricType]] | dict[str, list[str]]



import logging
import pkgutil
from pathlib import Path
from typing import Any, Callable

import degiro_connector.core.constants.timeouts as timeouts
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.core.helpers.lazy_loader import InitArgs, LazyLoader, Pair
from degiro_connector.core.models.model_connection import ModelConnection
from degiro_connector.core.models.model_session import ModelSession
from degiro_connector.trading.models.credentials import Credentials


class API:
    PKG_PATH = "degiro_connector.trading.actions"
    CLS_PREFIX = "Action"
    MOD_PREFIX = "action_"
    ROOT_PATH = Path(__file__).absolute().parent.parent.parent.resolve()

    @classmethod
    def build_action_list(cls) -> list[str]:
        # SETUP PATH
        path = cls.PKG_PATH
        path = str(Path(cls.ROOT_PATH, *path.split(".")).resolve())

        # BUILD MODULE LIST
        action_list = list()
        for module in pkgutil.iter_modules([path]):
            if (
                not module.ispkg
                and module.name[: len(cls.MOD_PREFIX)] == cls.MOD_PREFIX
            ):
                action_list.append(module.name[len(cls.MOD_PREFIX) :])

        return action_list

    @property
    def action_list(self) -> list[str]:
        return self._action_list

    @property
    def connection_storage(self) -> ModelConnection:
        return self._connection_storage

    @property
    def credentials(self) -> Credentials:
        return self._credentials

    @property
    def session_storage(self) -> ModelSession:
        return self._session_storage

    def load(
        self,
        action: str,
        init_args: InitArgs | None = None,
    ) -> object | None:
        logger = self._logger
        action_list = self._action_list

        if action not in action_list:
            logger.info("Not in action_list")
            logger.info("action : %s", action)
            logger.info("action_list : %s", action_list)
            return None

        # SETUP CLASS NAME
        cap_words_action = action.replace("_", " ").title().replace(" ", "")
        class_name = self.CLS_PREFIX + cap_words_action

        # SETUP PATHS
        module_path = self.PKG_PATH + "." + self.MOD_PREFIX + action

        # SETUP PAIR
        pair = Pair(
            module_path=module_path,
            class_name=class_name,
        )

        return LazyLoader.load_pair(pair=pair, init_args=init_args)

    def __init__(
        self,
        credentials: Credentials,
        connection_storage: ModelConnection | None = None,
        logger: logging.Logger | None = None,
        preload: bool = True,
        session_storage: ModelSession | None = None,
    ):
        self._credentials = credentials
        self._connection_storage = connection_storage or ModelConnection(
            timeout=timeouts.TRADING_TIMEOUT,
        )
        self._logger = logger or logging.getLogger(self.__module__)
        self._session_storage = session_storage or ModelSession(
            hooks=self._connection_storage.build_hooks()
        )
        self._action_list = self.build_action_list()

        if preload:
            self.setup_all_actions()

    def setup_all_actions(self):
        action_list = self._action_list
        for action in action_list:
            self.setup_one_action(action=action)

    def setup_one_action(self, action: str):
        logger = self._logger
        init_args = InitArgs(
            credentials=self._credentials,
            connection_storage=self._connection_storage,
            session_storage=self._session_storage,
        )
        action_instance = self.load(
            action=action,
            init_args=init_args,
        )
        if not isinstance(action_instance, AbstractAction):
            raise TypeError(
                "Not a `AbstractAction` : %s / %s " % (action, action_instance)
            )

        logger.debug("setup_one_action : %s", action)
        setattr(self, action, action_instance)

    def __getattr__(self, item) -> Callable[..., Any]:
        logger = self._logger
        logger.debug("CALLING __GETATTR__, on item : %s", item)
        if item in self._action_list:
            action = item
            self.setup_one_action(action=action)

            return getattr(self, action)

        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{item}'"
        )



import logging
from typing import Union

import requests
from google.protobuf import json_format

from degiro_connector.core.constants import urls
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.trading_pb2 import (
    Order,
)
from degiro_connector.core.abstracts.abstract_action import AbstractAction


class ActionCheckOrder(AbstractAction):
    ORDER_FILTER_MATCHING = {
        Order.OrderType.LIMIT: {
            "buySell",
            "orderType",
            "price",
            "productId",
            "size",
            "timeType",
        },
        Order.OrderType.STOP_LIMIT: {
            "buySell",
            "orderType",
            "price",
            "productId",
            "size",
            "stopPrice",
            "timeType",
        },
        Order.OrderType.MARKET: {
            "buySell",
            "orderType",
            "productId",
            "size",
            "timeType",
        },
        Order.OrderType.STOP_LOSS: {
            "buySell",
            "orderType",
            "productId",
            "size",
            "stopPrice",
            "timeType",
        },
    }

    @classmethod
    def order_to_api(cls, order: Order) -> dict[str, Union[float, int, str]]:
        # Build dict from message
        order_dict = json_format.MessageToDict(
            message=order,
            including_default_value_fields=True,
            preserving_proto_field_name=False,
            use_integers_for_enums=True,
            descriptor_pool=None,
            float_precision=None,
        )

        # Setup 'buySell'
        if order.action == order.Action.BUY:
            order_dict["buySell"] = "BUY"
        else:
            order_dict["buySell"] = "SELL"

        # Filter fields
        fields_to_keep = set()
        if order.order_type in cls.ORDER_FILTER_MATCHING:
            fields_to_keep = cls.ORDER_FILTER_MATCHING[order.order_type]
        else:
            raise AttributeError("Invalid `OrderType`.")

        filtered_order_dict = dict()
        for field in order_dict.keys() & fields_to_keep:
            filtered_order_dict[field] = order_dict[field]

        return filtered_order_dict

    @staticmethod
    def checking_response_to_grpc(payload: dict) -> Order.CheckingResponse:
        checking_response = Order.CheckingResponse()
        checking_response.response_datetime.GetCurrentTime()
        json_format.ParseDict(
            js_dict=payload["data"],
            message=checking_response,
            ignore_unknown_fields=True,
            descriptor_pool=None,
        )

        return checking_response

    @classmethod
    def check_order(
        cls,
        credentials: Credentials,
        order: Order,
        session_id: str,
        logger: logging.Logger | None = None,
        raw: bool = False,
        session: requests.Session | None = None,
    ) -> Union[Order.CheckingResponse, dict, None]:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.ORDER_CHECK
        url = f"{url};jsessionid={session_id}"

        params = {
            "intAccount": int_account,
            "sessionId": session_id,
        }

        order_dict = cls.order_to_api(order=order)

        request = requests.Request(
            method="POST",
            url=url,
            json=order_dict,
            params=params,
        )
        prepped = session.prepare_request(request)
        response_raw = None

        try:
            response_raw = session.send(prepped)
            response_raw.raise_for_status()
            response_dict = response_raw.json()
        except requests.HTTPError as e:
            status_code = getattr(response_raw, "status_code", "No status_code found.")
            text = getattr(response_raw, "text", "No text found.")
            logger.fatal(status_code)
            logger.fatal(text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

        if (
            isinstance(response_dict, dict)
            and "data" in response_dict
            and "confirmationId" in response_dict["data"]
        ):
            if raw is True:
                return response_dict
            else:
                return cls.checking_response_to_grpc(
                    payload=response_dict,
                )
        else:
            logger.fatal(response_raw)
            return None

    def call(
        self,
        order: Order,
        raw: bool = False,
    ) -> Union[Order.CheckingResponse, dict, None]:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        credentials = self.credentials
        session = self.session_storage.session
        logger = self.logger

        return self.check_order(
            credentials=credentials,
            order=order,
            session_id=session_id,
            logger=logger,
            raw=raw,
            session=session,
        )



class MessageRegistration:
    metric_name: str  # f"{product_id}.{metric_type}""
    reference: int

    def __init__(self, metric_name, reference):
        self.metric_name = metric_name
        self.reference = reference

    def __repr__(self) -> str:
        return f"`{self.reference}`:`{self.metric_name}`"


class MessageUnregistration:
    metric_name: str
    reference: int

    def __init__(self, metric_name, reference):
        self.metric_name = metric_name
        self.reference = reference

    def __repr__(self) -> str:
        return f"`{self.reference}`:`{self.metric_name}`"


class MessageNumeric:
    reference: int
    value: float

    def __init__(self, reference, value):
        self.reference = reference
        self.value = value

    def __repr__(self) -> str:
        return f"`{self.reference}`:`{self.value}`"


class MessageText:
    reference: int
    value: str

    def __init__(self, reference, value):
        self.reference = reference
        self.value = value

    def __repr__(self) -> str:
        return f"`{self.reference}`:`{self.value}`"


# A message from the list of messages received from Degiro's Quotecast API.
Message = MessageRegistration | MessageUnregistration | MessageNumeric | MessageText



import logging

import requests

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.favourite import FavouriteId, FavouriteName


class ActionCreateFavouriteList(AbstractAction):
    @classmethod
    def favourite_list_to_api(cls, name: str) -> dict[str, str]:
        return {"name": name}

    @classmethod
    def api_to_favourite_list_id(cls, response_dict: dict[str, int]) -> int:
        return response_dict["data"]

    @classmethod
    def create_favourite_list(
        cls,
        name: str,
        session_id: str,
        credentials: Credentials,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> int | None:
        """Create a favourite list.
        Args:
            name (str):
                New name of the favourite list.
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            Favourites: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.FAVOURITES_LIST

        params = {
            "intAccount": int_account,
            "sessionId": session_id,
        }

        json_obj = FavouriteName(name=name).model_dump(
            mode="python", by_alias=True, exclude_none=True
        )

        request = requests.Request(
            method="POST",
            url=url,
            params=params,
            json=json_obj,
        )
        prepped = session.prepare_request(request)

        favourite_list_id = None
        try:
            response = session.send(prepped)
            response.raise_for_status()
            favourite_list_id = FavouriteId.model_validate_json(
                json_data=response.text
            ).data
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

        return favourite_list_id

    def call(
        self,
        name: str,
    ) -> int | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.create_favourite_list(
            name=name,
            session_id=session_id,
            credentials=credentials,
            session=session,
            logger=logger,
        )



import logging
from typing import Union

import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.product_search import (
    ProductBatch,
    BondsRequest,
    ETFsRequest,
    FundsRequest,
    FuturesRequest,
    LeveragedsRequest,
    LookupRequest,
    OptionsRequest,
    StocksRequest,
    WarrantsRequest,
)

ProductRequest = (
    BondsRequest
    | ETFsRequest
    | FundsRequest
    | FuturesRequest
    | LeveragedsRequest
    | LookupRequest
    | OptionsRequest
    | StocksRequest
    | WarrantsRequest
)


class ActionProductSearch(AbstractAction):
    URL_MATCHING = {
        BondsRequest: urls.PRODUCT_SEARCH_BONDS,
        ETFsRequest: urls.PRODUCT_SEARCH_ETFS,
        FundsRequest: urls.PRODUCT_SEARCH_FUNDS,
        FuturesRequest: urls.PRODUCT_SEARCH_FUTURES,
        LeveragedsRequest: urls.PRODUCT_SEARCH_LEVERAGEDS,
        LookupRequest: urls.PRODUCT_SEARCH_LOOKUP,
        OptionsRequest: urls.PRODUCT_SEARCH_OPTIONS,
        StocksRequest: urls.PRODUCT_SEARCH_STOCKS,
        WarrantsRequest: urls.PRODUCT_SEARCH_WARRANTS,
    }

    @classmethod
    def product_search(
        cls,
        product_request: ProductRequest,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> Union[ProductBatch, dict, None]:
        """Search products.
        Args:
            request (StockList.Request):
                list of options that we want to retrieve from the endpoint.
                Example 1:
                    request = ProductBatch.LookupRequest(
                        search_text='APPLE',
                        limit=10,
                        offset=0,
                    )
                Example 2:
                    request = ProductBatch.StocksRequest(
                        index_id=5,
                        is_in_us_green_list=False,
                        stock_country_id=886,
                        offset=0,
                        limit=100,
                        require_total=True,
                        sort_columns='name',
                        sort_types='asc',
                    )
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            ProductBatch: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        url = cls.URL_MATCHING[type(product_request)]

        params = product_request.model_dump(
            mode="json", by_alias=True, exclude_none=True
        )

        if credentials.int_account is not None:
            params["intAccount"] = credentials.int_account

        params["sessionId"] = session_id

        http_request = requests.Request(method="GET", url=url, params=params)
        prepped = session.prepare_request(http_request)

        try:
            response = session.send(prepped)
            response.raise_for_status()

            if raw is True:
                product_search = loads(response.text)
            else:
                product_search = ProductBatch.model_validate_json(
                    json_data=response.text
                )

            return product_search
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        product_request: ProductRequest,
        raw: bool = False,
    ) -> Union[ProductBatch, dict, None]:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.product_search(
            product_request=product_request,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



import logging
from typing import Union

import requests
from google.protobuf import json_format

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.trading_pb2 import (
    Agenda,
)


class ActionGetAgenda(AbstractAction):
    @staticmethod
    def agenda_request_to_api(
        request: Agenda.Request,
    ) -> dict:
        request_dict = json_format.MessageToDict(
            message=request,
            including_default_value_fields=False,
            preserving_proto_field_name=False,
            use_integers_for_enums=True,
            descriptor_pool=None,
            float_precision=None,
        )
        request_dict["calendarType"] = (
            Agenda.CalendarType.Name(request.calendar_type).title().replace("_", "")
        )
        request_dict["offset"] = request.offset
        request_dict["orderByDesc"] = request.order_by_desc

        return request_dict

    @staticmethod
    def agenda_to_grpc(
        request: Agenda.Request,
        payload: dict,
    ) -> Agenda:
        agenda = Agenda()
        agenda.response_datetime.GetCurrentTime()
        agenda.calendar_type = request.calendar_type
        json_format.ParseDict(
            js_dict=payload,
            message=agenda,
            ignore_unknown_fields=True,
            descriptor_pool=None,
        )

        return agenda

    @classmethod
    def get_agenda(
        cls,
        request: Agenda.Request,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> Union[Agenda, dict, None]:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        url = urls.AGENDA
        params = cls.agenda_request_to_api(
            request=request,
        )
        params["intAccount"] = credentials.int_account
        params["sessionId"] = session_id

        req = requests.Request(method="GET", url=url, params=params)
        prepped = session.prepare_request(req)
        response_raw = None

        try:
            response_raw = session.send(prepped)
            response_raw.raise_for_status()
            response_dict = response_raw.json()

            if raw is True:
                return response_dict
            else:
                return cls.agenda_to_grpc(
                    request=request,
                    payload=response_dict,
                )
        except requests.HTTPError as e:
            status_code = getattr(response_raw, "status_code", "No status_code found.")
            text = getattr(response_raw, "text", "No text found.")
            logger.fatal(status_code)
            logger.fatal(text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        request: Agenda.Request,
        raw: bool = False,
    ) -> Union[Agenda, dict, None]:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_agenda(
            request=request,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



from datetime import datetime
from enum import Enum

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class Interval(str, Enum):
    PT15S = "PT15S"
    PT30S = "PT30S"
    PT1M = "PT1M"
    PT5M = "PT5M"
    PT15M = "PT15M"
    PT30M = "PT30M"
    PT60M = "PT60M"
    PT1H = "PT1H"
    P1D = "P1D"
    P1W = "P1W"
    P1M = "P1M"
    P3M = "P3M"
    P6M = "P6M"
    P1Y = "P1Y"
    P3Y = "P3Y"
    P5Y = "P5Y"
    P10Y = "P10Y"
    P50Y = "P50Y"
    YTD = "YTD"


class ChartRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    callback: str = Field(default="vwd.hchart.seriesRequestManager.sync_response")
    culture: str
    override: dict[str, str] = Field(default_factory=dict)
    format: str = Field(default="json")
    period: Interval
    requestid: str
    resolution: Interval
    series: list[str] = Field(default_factory=list)
    tz: str
    user_token: int | None = Field(default=None)


class Series(BaseModel):
    expires: datetime
    data: list | dict
    id: str
    type: str
    times: str | None = Field(default=None)


class Chart(BaseModel):
    end: str
    requestid: str
    resolution: str
    series: list[Series] = Field(default_factory=list)
    start: str




from google.protobuf import json_format
from google.protobuf.message import Message
from google.protobuf.struct_pb2 import Struct



def message_to_dict(message: Message) -> dict:
    return json_format.MessageToDict(
        message=message,
        including_default_value_fields=True,
        preserving_proto_field_name=True,
        use_integers_for_enums=True,
        descriptor_pool=None,
        float_precision=None,
    )


def struct_from_dict(json_dict: dict) -> Struct:
    return json_format.ParseDict(
        js_dict=json_dict,
        message=Struct(),
        ignore_unknown_fields=True,
        descriptor_pool=None,
    )



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel

class NewsRequest(BaseModel):
    isin: str
    limit: int = Field(default=10)
    offset: int = Field(default=0)
    languages: str


class NewsItem(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    brief: str | None = Field(default=None)
    category: str | None = Field(default=None)
    content: str
    date: datetime
    html_content: bool
    id: str
    isins: list[str]
    language: str
    last_updated: datetime | None = Field(default=None)
    picture_url: str | None = Field(default=None)
    provider: str
    source: str
    title: str


class NewsBatch(BaseModel):
    items: list[NewsItem] = Field(default_factory=list)
    offset: int
    total: int


class BatchWrapper(BaseModel):
    data: NewsBatch



from enum import Enum

from pydantic import BaseModel


class MetricType(str, Enum):
    LastDate = "LastDate"
    LastTime = "LastTime"
    LastPrice = "LastPrice"
    LastVolume = "LastVolume"
    A1Price = "A1Price"
    A2Price = "A2Price"
    A3Price = "A3Price"
    A4Price = "A4Price"
    A5Price = "A5Price"
    A6Price = "A6Price"
    A7Price = "A7Price"
    A8Price = "A8Price"
    A9Price = "A9Price"
    A10Price = "A10Price"
    A1Volume = "A1Volume"
    A2Volume = "A2Volume"
    A3Volume = "A3Volume"
    A4Volume = "A4Volume"
    A5Volume = "A5Volume"
    A6Volume = "A6Volume"
    A7Volume = "A7Volume"
    A8Volume = "A8Volume"
    A9Volume = "A9Volume"
    A10Volume = "A10Volume"
    A1Orders = "A1Orders"
    A2Orders = "A2Orders"
    A3Orders = "A3Orders"
    A4Orders = "A4Orders"
    A5Orders = "A5Orders"
    A6Orders = "A6Orders"
    A7Orders = "A7Orders"
    A8Orders = "A8Orders"
    A9Orders = "A9Orders"
    A10Orders = "A10Orders"
    B1Price = "B1Price"
    B2Price = "B2Price"
    B3Price = "B3Price"
    B4Price = "B4Price"
    B5Price = "B5Price"
    B6Price = "B6Price"
    B7Price = "B7Price"
    B8Price = "B8Price"
    B9Price = "B9Price"
    B10Price = "B10Price"
    B1Volume = "B1Volume"
    B2Volume = "B2Volume"
    B3Volume = "B3Volume"
    B4Volume = "B4Volume"
    B5Volume = "B5Volume"
    B6Volume = "B6Volume"
    B7Volume = "B7Volume"
    B8Volume = "B8Volume"
    B9Volume = "B9Volume"
    B10Volume = "B10Volume"
    B1Orders = "B1Orders"
    B2Orders = "B2Orders"
    B3Orders = "B3Orders"
    B4Orders = "B4Orders"
    B5Orders = "B5Orders"
    B6Orders = "B6Orders"
    B7Orders = "B7Orders"
    B8Orders = "B8Orders"
    B9Orders = "B9Orders"
    B10Orders = "B10Orders"
    BidVolume = "BidVolume"
    BidPrice = "BidPrice"
    AskVolume = "AskVolume"
    AskPrice = "AskPrice"
    OpenPrice = "OpenPrice"
    HighPrice = "HighPrice"
    LowPrice = "LowPrice"
    PreviousClosePrice = "PreviousClosePrice"
    PreviousCloseDate = "PreviousCloseDate"
    CumulativeVolume = "CumulativeVolume"
    FullName = "FullName"
    RelativeDifference = "RelativeDifference"
    AbsoluteDifference = "AbsoluteDifference"


class Metric(BaseModel):
    """Depicts one metric from a specific product: price, volume, ask, bid..."""

    metric_type: MetricType
    product_id: str
    value: str | float



from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class FavouriteName(BaseModel):
    name: str

class FavouriteId(BaseModel):
    data: int

class FavouritePosition(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    position: int
    list_id: int


class FavouriteItem(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    default: int | None = Field(default=None)
    id: int
    is_default: bool | None = Field(default=None)
    name: str | None = Field(default=None)


class FavouriteBatch(BaseModel):
    data: list[FavouriteItem] = Field(default_factory=list)



import datetime
import logging
from typing import Union

import requests

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.trading_pb2 import (
    PositionReport,
)


class ActionGetPositionReport(AbstractAction):
    @staticmethod
    def position_report_request_to_api(
        request: PositionReport.Request,
    ) -> dict:
        request_dict = dict()
        request_dict["country"] = request.country
        request_dict["lang"] = request.lang
        request_dict["toDate"] = datetime.datetime(
            year=request.to_date.year,
            month=request.to_date.month,
            day=request.to_date.day,
        ).strftime("%d/%m/%Y")

        return request_dict

    @staticmethod
    def position_report_to_grpc(
        request: PositionReport.Request,
        payload: str,
    ) -> PositionReport:
        position_report = PositionReport()
        position_report.response_datetime.GetCurrentTime()
        position_report.content = payload
        position_report.format = request.format

        return position_report

    @classmethod
    def get_position_report(
        cls,
        request: PositionReport.Request,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> Union[PositionReport, str, None]:
        """Retrieve information about the account in a specific format.
        Args:
            request (PositionReport.Request):
                list of options that we want to retrieve from the endpoint.
                Example :
                    to_date = PositionReport.Request.Date(
                        year=2020,
                        month=10,
                        day=15,
                    )
                    request = PositionReport.Request(
                        format=PositionReport.Format.CSV,
                        country='FR',
                        lang='fr',
                        to_date=to_date,
                    )
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            PositionReport: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        format = PositionReport.Format.Name(request.format)
        url = f"{urls.POSITION_REPORT}/{format}"
        params = cls.position_report_request_to_api(
            request=request,
        )
        params["intAccount"] = credentials.int_account
        params["sessionId"] = session_id

        req = requests.Request(method="GET", url=url, params=params)
        prepped = session.prepare_request(req)
        response_raw = None

        try:
            response_raw = session.send(prepped)
            response_raw.raise_for_status()
            response_text = response_raw.text

            if raw is True:
                return response_text
            else:
                return cls.position_report_to_grpc(
                    request=request,
                    payload=response_text,
                )
        except requests.HTTPError as e:
            status_code = getattr(response_raw, "status_code", "No status_code found.")
            text = getattr(response_raw, "text", "No text found.")
            logger.fatal(status_code)
            logger.fatal(text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        request: PositionReport.Request,
        raw: bool = False,
    ) -> Union[PositionReport, str, None]:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_position_report(
            request=request,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: degiro_connector/trading/models/trading.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-degiro_connector/trading/models/trading.proto\x12\x18\x64\x65giro_connector.trading\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x8b\x01\n\x0b\x43redentials\x12\x13\n\x0bint_account\x18\x01 \x01(\x05\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x10\n\x08password\x18\x03 \x01(\t\x12\x19\n\x0ftotp_secret_key\x18\x04 \x01(\tH\x00\x12\x1b\n\x11one_time_password\x18\x05 \x01(\x05H\x00\x42\x0b\n\toneof_2fa\"\x94\n\n\x05Order\x12\x36\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32&.degiro_connector.trading.Order.Action\x12\n\n\x02id\x18\x02 \x01(\t\x12=\n\norder_type\x18\x03 \x01(\x0e\x32).degiro_connector.trading.Order.OrderType\x12\r\n\x05price\x18\x04 \x01(\x01\x12\x12\n\nstop_price\x18\x05 \x01(\x01\x12\x12\n\nproduct_id\x18\x06 \x01(\x05\x12\x0c\n\x04size\x18\x07 \x01(\x01\x12;\n\ttime_type\x18\x08 \x01(\x0e\x32(.degiro_connector.trading.Order.TimeType\x12\x15\n\rcontract_size\x18\t \x01(\x01\x12\x15\n\rcontract_type\x18\n \x01(\x05\x12\x10\n\x08\x63urrency\x18\x0b \x01(\t\x12\x0c\n\x04hour\x18\x0c \x01(\t\x12\x14\n\x0cis_deletable\x18\r \x01(\x08\x12\x15\n\ris_modifiable\x18\x0e \x01(\x08\x12\x0f\n\x07product\x18\x0f \x01(\t\x12\x10\n\x08quantity\x18\x10 \x01(\x01\x12\x19\n\x11total_order_value\x18\x11 \x01(\x01\x12\x16\n\x0eretained_order\x18\x12 \x01(\x08\x12\x18\n\x10sent_to_exchange\x18\x13 \x01(\x08\x1a\x84\x04\n\x10\x43heckingResponse\x12\x17\n\x0f\x63onfirmation_id\x18\x01 \x01(\t\x12\x16\n\x0e\x66ree_space_new\x18\x02 \x01(\x01\x12\x35\n\x11response_datetime\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\x10transaction_fees\x18\x04 \x03(\x0b\x32\x17.google.protobuf.Struct\x12:\n\x19transaction_opposite_fees\x18\x05 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x32\n\x11transaction_taxes\x18\x06 \x03(\x0b\x32\x17.google.protobuf.Struct\x12?\n\x1etransaction_auto_fx_surcharges\x18\x07 \x03(\x0b\x32\x17.google.protobuf.Struct\x12H\n\'transaction_auto_fx_opposite_surcharges\x18\x08 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x1f\n\x17\x61uto_fx_conversion_rate\x18\t \x01(\x01\x12\x17\n\x0ftransaction_fee\x18\n \x01(\x01\x12 \n\x18show_ex_ante_report_link\x18\x0b \x01(\x08\x1a_\n\x14\x43onfirmationResponse\x12\x10\n\x08order_id\x18\x01 \x01(\t\x12\x35\n\x11response_datetime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x1b\n\x06\x41\x63tion\x12\x07\n\x03\x42UY\x10\x00\x12\x08\n\x04SELL\x10\x01\"A\n\tOrderType\x12\t\n\x05LIMIT\x10\x00\x12\x0e\n\nSTOP_LIMIT\x10\x01\x12\n\n\x06MARKET\x10\x02\x12\r\n\tSTOP_LOSS\x10\x03\"S\n\x08TimeType\x12\r\n\tUNKNOWN_0\x10\x00\x12\x11\n\rGOOD_TILL_DAY\x10\x01\x12\r\n\tUNKNOWN_2\x10\x02\x12\x16\n\x12GOOD_TILL_CANCELED\x10\x03\"\xd9\n\n\x06Update\x12\x35\n\x11response_datetime\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x37\n\x06\x61lerts\x18\x02 \x01(\x0b\x32\'.degiro_connector.trading.Update.Alerts\x12>\n\ncash_funds\x18\x03 \x01(\x0b\x32*.degiro_connector.trading.Update.CashFunds\x12L\n\x11historical_orders\x18\x04 \x01(\x0b\x32\x31.degiro_connector.trading.Update.HistoricalOrders\x12\x37\n\x06orders\x18\x05 \x01(\x0b\x32\'.degiro_connector.trading.Update.Orders\x12=\n\tportfolio\x18\x06 \x01(\x0b\x32*.degiro_connector.trading.Update.Portfolio\x12H\n\x0ftotal_portfolio\x18\x07 \x01(\x0b\x32/.degiro_connector.trading.Update.TotalPortfolio\x12\x43\n\x0ctransactions\x18\x08 \x01(\x0b\x32-.degiro_connector.trading.Update.Transactions\x1aX\n\x07Request\x12\x37\n\x06option\x18\x01 \x01(\x0e\x32\'.degiro_connector.trading.Update.Option\x12\x14\n\x0clast_updated\x18\x02 \x01(\x05\x1aG\n\x0bRequestList\x12\x38\n\x06values\x18\x01 \x03(\x0b\x32(.degiro_connector.trading.Update.Request\x1aG\n\x06\x41lerts\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12\'\n\x06values\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x1aJ\n\tCashFunds\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12\'\n\x06values\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x1aY\n\x10HistoricalOrders\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12/\n\x06values\x18\x02 \x03(\x0b\x32\x1f.degiro_connector.trading.Order\x1aO\n\x06Orders\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12/\n\x06values\x18\x02 \x03(\x0b\x32\x1f.degiro_connector.trading.Order\x1aJ\n\tPortfolio\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12\'\n\x06values\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x1aO\n\x0eTotalPortfolio\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12\'\n\x06values\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x1aM\n\x0cTransactions\x12\x14\n\x0clast_updated\x18\x01 \x01(\x05\x12\'\n\x06values\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\"z\n\x06Option\x12\n\n\x06\x41LERTS\x10\x00\x12\r\n\tCASHFUNDS\x10\x01\x12\x14\n\x10HISTORICALORDERS\x10\x02\x12\n\n\x06ORDERS\x10\x03\x12\r\n\tPORTFOLIO\x10\x04\x12\x12\n\x0eTOTALPORTFOLIO\x10\x05\x12\x10\n\x0cTRANSACTIONS\x10\x06\"\xbd\x02\n\rOrdersHistory\x12\'\n\x06values\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x35\n\x11response_datetime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xcb\x01\n\x07Request\x12G\n\tfrom_date\x18\x01 \x01(\x0b\x32\x34.degiro_connector.trading.OrdersHistory.Request.Date\x12\x45\n\x07to_date\x18\x02 \x01(\x0b\x32\x34.degiro_connector.trading.OrdersHistory.Request.Date\x1a\x30\n\x04\x44\x61te\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05\"\xf4\x02\n\x13TransactionsHistory\x12\'\n\x06values\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x35\n\x11response_datetime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xfc\x01\n\x07Request\x12M\n\tfrom_date\x18\x01 \x01(\x0b\x32:.degiro_connector.trading.TransactionsHistory.Request.Date\x12K\n\x07to_date\x18\x02 \x01(\x0b\x32:.degiro_connector.trading.TransactionsHistory.Request.Date\x12#\n\x1bgroup_transactions_by_order\x18\x03 \x01(\x08\x1a\x30\n\x04\x44\x61te\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05\"\xc3\x02\n\x0f\x41\x63\x63ountOverview\x12\'\n\x06values\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x35\n\x11response_datetime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xcf\x01\n\x07Request\x12I\n\tfrom_date\x18\x01 \x01(\x0b\x32\x36.degiro_connector.trading.AccountOverview.Request.Date\x12G\n\x07to_date\x18\x02 \x01(\x0b\x32\x36.degiro_connector.trading.AccountOverview.Request.Date\x1a\x30\n\x04\x44\x61te\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05\"\xc5\x0e\n\rProductSearch\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12)\n\x08products\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x35\n\x11response_datetime\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05total\x18\x04 \x01(\x05\x1a\x31\n\x06\x43onfig\x12\'\n\x06values\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x1a\xba\x01\n\x0cRequestBonds\x12\x1b\n\x13\x62ond_issuer_type_id\x18\x01 \x01(\x05\x12\x18\n\x10\x62ond_exchange_id\x18\x02 \x01(\x05\x12\x13\n\x0bsearch_text\x18\x03 \x01(\t\x12\x0e\n\x06offset\x18\x04 \x01(\x05\x12\r\n\x05limit\x18\x05 \x01(\x05\x12\x15\n\rrequire_total\x18\x06 \x01(\x08\x12\x14\n\x0csort_columns\x18\x07 \x01(\t\x12\x12\n\nsort_types\x18\x08 \x01(\t\x1a\xd7\x01\n\x0bRequestETFs\x12\x14\n\x0cpopular_only\x18\x01 \x01(\x08\x12\x1d\n\x15input_aggregate_types\x18\x02 \x01(\t\x12\x1e\n\x16input_aggregate_values\x18\x03 \x01(\t\x12\x13\n\x0bsearch_text\x18\x04 \x01(\t\x12\x0e\n\x06offset\x18\x05 \x01(\x05\x12\r\n\x05limit\x18\x06 \x01(\x05\x12\x15\n\rrequire_total\x18\x07 \x01(\x08\x12\x14\n\x0csort_columns\x18\x08 \x01(\t\x12\x12\n\nsort_types\x18\t \x01(\t\x1a\x83\x01\n\x0cRequestFunds\x12\x13\n\x0bsearch_text\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x15\n\rrequire_total\x18\x04 \x01(\x08\x12\x14\n\x0csort_columns\x18\x05 \x01(\t\x12\x12\n\nsort_types\x18\x06 \x01(\t\x1a\xba\x01\n\x0eRequestFutures\x12\x1a\n\x12\x66uture_exchange_id\x18\x01 \x01(\x05\x12\x17\n\x0funderlying_isin\x18\x02 \x01(\t\x12\x13\n\x0bsearch_text\x18\x03 \x01(\t\x12\x0e\n\x06offset\x18\x04 \x01(\x05\x12\r\n\x05limit\x18\x05 \x01(\x05\x12\x15\n\rrequire_total\x18\x06 \x01(\x08\x12\x14\n\x0csort_columns\x18\x07 \x01(\t\x12\x12\n\nsort_types\x18\x08 \x01(\t\x1a\xdd\x01\n\x11RequestLeverageds\x12\x14\n\x0cpopular_only\x18\x01 \x01(\x08\x12\x1d\n\x15input_aggregate_types\x18\x02 \x01(\t\x12\x1e\n\x16input_aggregate_values\x18\x03 \x01(\t\x12\x13\n\x0bsearch_text\x18\x04 \x01(\t\x12\x0e\n\x06offset\x18\x05 \x01(\x05\x12\r\n\x05limit\x18\x06 \x01(\x05\x12\x15\n\rrequire_total\x18\x07 \x01(\x08\x12\x14\n\x0csort_columns\x18\x08 \x01(\t\x12\x12\n\nsort_types\x18\t \x01(\t\x1a\\\n\rRequestLookup\x12\x13\n\x0bsearch_text\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x0e\n\x06offset\x18\x03 \x01(\x05\x12\x17\n\x0fproduct_type_id\x18\x04 \x01(\x05\x1a\xf9\x01\n\x0eRequestOptions\x12\x1d\n\x15input_aggregate_types\x18\x01 \x01(\t\x12\x1e\n\x16input_aggregate_values\x18\x02 \x01(\t\x12\x1a\n\x12option_exchange_id\x18\x03 \x01(\x05\x12\x17\n\x0funderlying_isin\x18\x04 \x01(\t\x12\x13\n\x0bsearch_text\x18\x05 \x01(\t\x12\x0e\n\x06offset\x18\x06 \x01(\x05\x12\r\n\x05limit\x18\x07 \x01(\x05\x12\x15\n\rrequire_total\x18\x08 \x01(\x08\x12\x14\n\x0csort_columns\x18\t \x01(\t\x12\x12\n\nsort_types\x18\n \x01(\t\x1a\xe2\x01\n\rRequestStocks\x12\x1b\n\x13is_in_us_green_list\x18\x01 \x01(\x08\x12\x10\n\x08index_id\x18\x02 \x01(\x05\x12\x13\n\x0b\x65xchange_id\x18\x03 \x01(\x05\x12\x18\n\x10stock_country_id\x18\x04 \x01(\x05\x12\x13\n\x0bsearch_text\x18\x05 \x01(\t\x12\x0e\n\x06offset\x18\x06 \x01(\x05\x12\r\n\x05limit\x18\x07 \x01(\x05\x12\x15\n\rrequire_total\x18\x08 \x01(\x08\x12\x14\n\x0csort_columns\x18\t \x01(\t\x12\x12\n\nsort_types\x18\n \x01(\t\x1a\x86\x01\n\x0fRequestWarrants\x12\x13\n\x0bsearch_text\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x15\n\rrequire_total\x18\x04 \x01(\x08\x12\x14\n\x0csort_columns\x18\x05 \x01(\t\x12\x12\n\nsort_types\x18\x06 \x01(\t\"\xc9\x01\n\nFavourites\x12\x39\n\x06values\x18\x01 \x03(\x0b\x32).degiro_connector.trading.Favourites.List\x12\x35\n\x11response_datetime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1aI\n\x04List\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nis_default\x18\x03 \x01(\x08\x12\x13\n\x0bproduct_ids\x18\x04 \x03(\x05\"T\n\x0cProductsInfo\x12\'\n\x06values\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x1a\x1b\n\x07Request\x12\x10\n\x08products\x18\x01 \x03(\x05\"8\n\rCompanyRatios\x12\'\n\x06values\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\"9\n\x0e\x43ompanyProfile\x12\'\n\x06values\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\">\n\x13\x46inancialStatements\x12\'\n\x06values\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x90\x01\n\nLatestNews\x12&\n\x05items\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05total\x18\x03 \x01(\x05\x1a;\n\x07Request\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12\x11\n\tlanguages\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\"W\n\x0eTopNewsPreview\x12&\n\x05items\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05total\x18\x03 \x01(\x05\"\xa1\x01\n\rNewsByCompany\x12&\n\x05items\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05total\x18\x03 \x01(\x05\x1aI\n\x07Request\x12\x0c\n\x04isin\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x0e\n\x06offset\x18\x03 \x01(\x05\x12\x11\n\tlanguages\x18\x04 \x01(\t\"\x87\x04\n\x11\x43\x61shAccountReport\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12\x42\n\x06\x66ormat\x18\x02 \x01(\x0e\x32\x32.degiro_connector.trading.CashAccountReport.Format\x12\x35\n\x11response_datetime\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xb6\x02\n\x07Request\x12\x42\n\x06\x66ormat\x18\x01 \x01(\x0e\x32\x32.degiro_connector.trading.CashAccountReport.Format\x12\x0f\n\x07\x63ountry\x18\x02 \x01(\t\x12\x0c\n\x04lang\x18\x03 \x01(\t\x12K\n\tfrom_date\x18\x04 \x01(\x0b\x32\x38.degiro_connector.trading.CashAccountReport.Request.Date\x12I\n\x07to_date\x18\x05 \x01(\x0b\x32\x38.degiro_connector.trading.CashAccountReport.Request.Date\x1a\x30\n\x04\x44\x61te\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05\"-\n\x06\x46ormat\x12\x07\n\x03\x43SV\x10\x00\x12\x08\n\x04HTML\x10\x01\x12\x07\n\x03PDF\x10\x02\x12\x07\n\x03XLS\x10\x03\"\x95\x05\n\x06\x41genda\x12\x44\n\rcalendar_type\x18\x01 \x01(\x0e\x32-.degiro_connector.trading.Agenda.CalendarType\x12&\n\x05items\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x0e\n\x06offset\x18\x03 \x01(\x05\x12\r\n\x05total\x18\x04 \x01(\x05\x12\x35\n\x11response_datetime\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xb4\x02\n\x07Request\x12\x44\n\rcalendar_type\x18\x01 \x01(\x0e\x32-.degiro_connector.trading.Agenda.CalendarType\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x15\n\rorder_by_desc\x18\x04 \x01(\x08\x12.\n\nstart_date\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_date\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x14\n\x0c\x63ompany_name\x18\x07 \x01(\t\x12\x11\n\tcountries\x18\x08 \x01(\t\x12\x17\n\x0f\x63lassifications\x18\t \x01(\t\x12\r\n\x05units\x18\n \x01(\t\"\x8f\x01\n\x0c\x43\x61lendarType\x12\x15\n\x11\x44IVIDEND_CALENDAR\x10\x00\x12\x15\n\x11\x45\x43ONOMIC_CALENDAR\x10\x01\x12\x15\n\x11\x45\x41RNINGS_CALENDAR\x10\x02\x12\x14\n\x10HOLIDAY_CALENDAR\x10\x03\x12\x10\n\x0cIPO_CALENDAR\x10\x04\x12\x12\n\x0eSPLIT_CALENDAR\x10\x05\"\xcc\x01\n\x12\x45stimatesSummaries\x12\'\n\x06\x61nnual\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x10\n\x08\x63urrency\x18\x02 \x01(\t\x12(\n\x07interim\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x15\n\rlastRetrieved\x18\x04 \x01(\t\x12\x13\n\x0blastUpdated\x18\x05 \x01(\t\x12\x18\n\x10preferredMeasure\x18\x06 \x01(\t\x12\x0b\n\x03ric\x18\x07 \x01(\t\"\xae\x03\n\x0ePositionReport\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12?\n\x06\x66ormat\x18\x02 \x01(\x0e\x32/.degiro_connector.trading.PositionReport.Format\x12\x35\n\x11response_datetime\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xe3\x01\n\x07Request\x12?\n\x06\x66ormat\x18\x01 \x01(\x0e\x32/.degiro_connector.trading.PositionReport.Format\x12\x0f\n\x07\x63ountry\x18\x02 \x01(\t\x12\x0c\n\x04lang\x18\x03 \x01(\t\x12\x46\n\x07to_date\x18\x05 \x01(\x0b\x32\x35.degiro_connector.trading.PositionReport.Request.Date\x1a\x30\n\x04\x44\x61te\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05\"-\n\x06\x46ormat\x12\x07\n\x03\x43SV\x10\x00\x12\x08\n\x04HTML\x10\x01\x12\x07\n\x03PDF\x10\x02\x12\x07\n\x03XLS\x10\x03\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'degiro_connector.trading.models.trading_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CREDENTIALS._serialized_start=139
  _CREDENTIALS._serialized_end=278
  _ORDER._serialized_start=281
  _ORDER._serialized_end=1581
  _ORDER_CHECKINGRESPONSE._serialized_start=787
  _ORDER_CHECKINGRESPONSE._serialized_end=1303
  _ORDER_CONFIRMATIONRESPONSE._serialized_start=1305
  _ORDER_CONFIRMATIONRESPONSE._serialized_end=1400
  _ORDER_ACTION._serialized_start=1402
  _ORDER_ACTION._serialized_end=1429
  _ORDER_ORDERTYPE._serialized_start=1431
  _ORDER_ORDERTYPE._serialized_end=1496
  _ORDER_TIMETYPE._serialized_start=1498
  _ORDER_TIMETYPE._serialized_end=1581
  _UPDATE._serialized_start=1584
  _UPDATE._serialized_end=2953
  _UPDATE_REQUEST._serialized_start=2111
  _UPDATE_REQUEST._serialized_end=2199
  _UPDATE_REQUESTLIST._serialized_start=2201
  _UPDATE_REQUESTLIST._serialized_end=2272
  _UPDATE_ALERTS._serialized_start=2274
  _UPDATE_ALERTS._serialized_end=2345
  _UPDATE_CASHFUNDS._serialized_start=2347
  _UPDATE_CASHFUNDS._serialized_end=2421
  _UPDATE_HISTORICALORDERS._serialized_start=2423
  _UPDATE_HISTORICALORDERS._serialized_end=2512
  _UPDATE_ORDERS._serialized_start=2514
  _UPDATE_ORDERS._serialized_end=2593
  _UPDATE_PORTFOLIO._serialized_start=2595
  _UPDATE_PORTFOLIO._serialized_end=2669
  _UPDATE_TOTALPORTFOLIO._serialized_start=2671
  _UPDATE_TOTALPORTFOLIO._serialized_end=2750
  _UPDATE_TRANSACTIONS._serialized_start=2752
  _UPDATE_TRANSACTIONS._serialized_end=2829
  _UPDATE_OPTION._serialized_start=2831
  _UPDATE_OPTION._serialized_end=2953
  _ORDERSHISTORY._serialized_start=2956
  _ORDERSHISTORY._serialized_end=3273
  _ORDERSHISTORY_REQUEST._serialized_start=3070
  _ORDERSHISTORY_REQUEST._serialized_end=3273
  _ORDERSHISTORY_REQUEST_DATE._serialized_start=3225
  _ORDERSHISTORY_REQUEST_DATE._serialized_end=3273
  _TRANSACTIONSHISTORY._serialized_start=3276
  _TRANSACTIONSHISTORY._serialized_end=3648
  _TRANSACTIONSHISTORY_REQUEST._serialized_start=3396
  _TRANSACTIONSHISTORY_REQUEST._serialized_end=3648
  _TRANSACTIONSHISTORY_REQUEST_DATE._serialized_start=3225
  _TRANSACTIONSHISTORY_REQUEST_DATE._serialized_end=3273
  _ACCOUNTOVERVIEW._serialized_start=3651
  _ACCOUNTOVERVIEW._serialized_end=3974
  _ACCOUNTOVERVIEW_REQUEST._serialized_start=3767
  _ACCOUNTOVERVIEW_REQUEST._serialized_end=3974
  _ACCOUNTOVERVIEW_REQUEST_DATE._serialized_start=3225
  _ACCOUNTOVERVIEW_REQUEST_DATE._serialized_end=3273
  _PRODUCTSEARCH._serialized_start=3977
  _PRODUCTSEARCH._serialized_end=5838
  _PRODUCTSEARCH_CONFIG._serialized_start=4123
  _PRODUCTSEARCH_CONFIG._serialized_end=4172
  _PRODUCTSEARCH_REQUESTBONDS._serialized_start=4175
  _PRODUCTSEARCH_REQUESTBONDS._serialized_end=4361
  _PRODUCTSEARCH_REQUESTETFS._serialized_start=4364
  _PRODUCTSEARCH_REQUESTETFS._serialized_end=4579
  _PRODUCTSEARCH_REQUESTFUNDS._serialized_start=4582
  _PRODUCTSEARCH_REQUESTFUNDS._serialized_end=4713
  _PRODUCTSEARCH_REQUESTFUTURES._serialized_start=4716
  _PRODUCTSEARCH_REQUESTFUTURES._serialized_end=4902
  _PRODUCTSEARCH_REQUESTLEVERAGEDS._serialized_start=4905
  _PRODUCTSEARCH_REQUESTLEVERAGEDS._serialized_end=5126
  _PRODUCTSEARCH_REQUESTLOOKUP._serialized_start=5128
  _PRODUCTSEARCH_REQUESTLOOKUP._serialized_end=5220
  _PRODUCTSEARCH_REQUESTOPTIONS._serialized_start=5223
  _PRODUCTSEARCH_REQUESTOPTIONS._serialized_end=5472
  _PRODUCTSEARCH_REQUESTSTOCKS._serialized_start=5475
  _PRODUCTSEARCH_REQUESTSTOCKS._serialized_end=5701
  _PRODUCTSEARCH_REQUESTWARRANTS._serialized_start=5704
  _PRODUCTSEARCH_REQUESTWARRANTS._serialized_end=5838
  _FAVOURITES._serialized_start=5841
  _FAVOURITES._serialized_end=6042
  _FAVOURITES_LIST._serialized_start=5969
  _FAVOURITES_LIST._serialized_end=6042
  _PRODUCTSINFO._serialized_start=6044
  _PRODUCTSINFO._serialized_end=6128
  _PRODUCTSINFO_REQUEST._serialized_start=6101
  _PRODUCTSINFO_REQUEST._serialized_end=6128
  _COMPANYRATIOS._serialized_start=6130
  _COMPANYRATIOS._serialized_end=6186
  _COMPANYPROFILE._serialized_start=6188
  _COMPANYPROFILE._serialized_end=6245
  _FINANCIALSTATEMENTS._serialized_start=6247
  _FINANCIALSTATEMENTS._serialized_end=6309
  _LATESTNEWS._serialized_start=6312
  _LATESTNEWS._serialized_end=6456
  _LATESTNEWS_REQUEST._serialized_start=6397
  _LATESTNEWS_REQUEST._serialized_end=6456
  _TOPNEWSPREVIEW._serialized_start=6458
  _TOPNEWSPREVIEW._serialized_end=6545
  _NEWSBYCOMPANY._serialized_start=6548
  _NEWSBYCOMPANY._serialized_end=6709
  _NEWSBYCOMPANY_REQUEST._serialized_start=6636
  _NEWSBYCOMPANY_REQUEST._serialized_end=6709
  _CASHACCOUNTREPORT._serialized_start=6712
  _CASHACCOUNTREPORT._serialized_end=7231
  _CASHACCOUNTREPORT_REQUEST._serialized_start=6874
  _CASHACCOUNTREPORT_REQUEST._serialized_end=7184
  _CASHACCOUNTREPORT_REQUEST_DATE._serialized_start=3225
  _CASHACCOUNTREPORT_REQUEST_DATE._serialized_end=3273
  _CASHACCOUNTREPORT_FORMAT._serialized_start=7186
  _CASHACCOUNTREPORT_FORMAT._serialized_end=7231
  _AGENDA._serialized_start=7234
  _AGENDA._serialized_end=7895
  _AGENDA_REQUEST._serialized_start=7441
  _AGENDA_REQUEST._serialized_end=7749
  _AGENDA_CALENDARTYPE._serialized_start=7752
  _AGENDA_CALENDARTYPE._serialized_end=7895
  _ESTIMATESSUMMARIES._serialized_start=7898
  _ESTIMATESSUMMARIES._serialized_end=8102
  _POSITIONREPORT._serialized_start=8105
  _POSITIONREPORT._serialized_end=8535
  _POSITIONREPORT_REQUEST._serialized_start=8261
  _POSITIONREPORT_REQUEST._serialized_end=8488
  _POSITIONREPORT_REQUEST_DATE._serialized_start=3225
  _POSITIONREPORT_REQUEST_DATE._serialized_end=3273
  _POSITIONREPORT_FORMAT._serialized_start=7186
  _POSITIONREPORT_FORMAT._serialized_end=7231
# @@protoc_insertion_point(module_scope)



import logging
from typing import Union

import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.news import BatchWrapper, NewsBatch, NewsRequest


class ActionGetNewsByCompany(AbstractAction):
    @classmethod
    def get_news_by_company(
        cls,
        news_request: NewsRequest,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> Union[NewsBatch, dict, None]:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        url = urls.NEWS_BY_COMPANY

        params = news_request.model_dump(
            mode="python", by_alias=True, exclude_none=True
        )
        params["intAccount"] = credentials.int_account
        params["sessionId"] = session_id

        http_request = requests.Request(method="GET", url=url, params=params)
        prepped = session.prepare_request(http_request)
        prepped.headers["cookie"] = "JSESSIONID=" + session_id

        response_raw = None

        try:
            response = session.send(prepped)
            response.raise_for_status()

            if raw is True:
                company_news = loads(response.text)
            else:
                company_news = BatchWrapper.model_validate_json(
                    json_data=response.text
                ).data

            return company_news
        except Exception as e:
            logger.fatal(e)
            logger.fatal(response_raw)
            return None

    def call(
        self,
        news_request: NewsRequest,
        raw: bool = False,
    ) -> Union[NewsBatch, dict, None]:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_news_by_company(
            news_request=news_request,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AccountOverview(_message.Message):
    __slots__ = ["response_datetime", "values"]
    class Request(_message.Message):
        __slots__ = ["from_date", "to_date"]
        class Date(_message.Message):
            __slots__ = ["day", "month", "year"]
            DAY_FIELD_NUMBER: _ClassVar[int]
            MONTH_FIELD_NUMBER: _ClassVar[int]
            YEAR_FIELD_NUMBER: _ClassVar[int]
            day: int
            month: int
            year: int
            def __init__(self, year: _Optional[int] = ..., month: _Optional[int] = ..., day: _Optional[int] = ...) -> None: ...
        FROM_DATE_FIELD_NUMBER: _ClassVar[int]
        TO_DATE_FIELD_NUMBER: _ClassVar[int]
        from_date: AccountOverview.Request.Date
        to_date: AccountOverview.Request.Date
        def __init__(self, from_date: _Optional[_Union[AccountOverview.Request.Date, _Mapping]] = ..., to_date: _Optional[_Union[AccountOverview.Request.Date, _Mapping]] = ...) -> None: ...
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    response_datetime: _timestamp_pb2.Timestamp
    values: _struct_pb2.Struct
    def __init__(self, values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class Agenda(_message.Message):
    __slots__ = ["calendar_type", "items", "offset", "response_datetime", "total"]
    class CalendarType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class Request(_message.Message):
        __slots__ = ["calendar_type", "classifications", "company_name", "countries", "end_date", "limit", "offset", "order_by_desc", "start_date", "units"]
        CALENDAR_TYPE_FIELD_NUMBER: _ClassVar[int]
        CLASSIFICATIONS_FIELD_NUMBER: _ClassVar[int]
        COMPANY_NAME_FIELD_NUMBER: _ClassVar[int]
        COUNTRIES_FIELD_NUMBER: _ClassVar[int]
        END_DATE_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        ORDER_BY_DESC_FIELD_NUMBER: _ClassVar[int]
        START_DATE_FIELD_NUMBER: _ClassVar[int]
        UNITS_FIELD_NUMBER: _ClassVar[int]
        calendar_type: Agenda.CalendarType
        classifications: str
        company_name: str
        countries: str
        end_date: _timestamp_pb2.Timestamp
        limit: int
        offset: int
        order_by_desc: bool
        start_date: _timestamp_pb2.Timestamp
        units: str
        def __init__(self, calendar_type: _Optional[_Union[Agenda.CalendarType, str]] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., order_by_desc: bool = ..., start_date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., company_name: _Optional[str] = ..., countries: _Optional[str] = ..., classifications: _Optional[str] = ..., units: _Optional[str] = ...) -> None: ...
    CALENDAR_TYPE_FIELD_NUMBER: _ClassVar[int]
    DIVIDEND_CALENDAR: Agenda.CalendarType
    EARNINGS_CALENDAR: Agenda.CalendarType
    ECONOMIC_CALENDAR: Agenda.CalendarType
    HOLIDAY_CALENDAR: Agenda.CalendarType
    IPO_CALENDAR: Agenda.CalendarType
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    SPLIT_CALENDAR: Agenda.CalendarType
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    calendar_type: Agenda.CalendarType
    items: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    offset: int
    response_datetime: _timestamp_pb2.Timestamp
    total: int
    def __init__(self, calendar_type: _Optional[_Union[Agenda.CalendarType, str]] = ..., items: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., offset: _Optional[int] = ..., total: _Optional[int] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class CashAccountReport(_message.Message):
    __slots__ = ["content", "format", "response_datetime"]
    class Format(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class Request(_message.Message):
        __slots__ = ["country", "format", "from_date", "lang", "to_date"]
        class Date(_message.Message):
            __slots__ = ["day", "month", "year"]
            DAY_FIELD_NUMBER: _ClassVar[int]
            MONTH_FIELD_NUMBER: _ClassVar[int]
            YEAR_FIELD_NUMBER: _ClassVar[int]
            day: int
            month: int
            year: int
            def __init__(self, year: _Optional[int] = ..., month: _Optional[int] = ..., day: _Optional[int] = ...) -> None: ...
        COUNTRY_FIELD_NUMBER: _ClassVar[int]
        FORMAT_FIELD_NUMBER: _ClassVar[int]
        FROM_DATE_FIELD_NUMBER: _ClassVar[int]
        LANG_FIELD_NUMBER: _ClassVar[int]
        TO_DATE_FIELD_NUMBER: _ClassVar[int]
        country: str
        format: CashAccountReport.Format
        from_date: CashAccountReport.Request.Date
        lang: str
        to_date: CashAccountReport.Request.Date
        def __init__(self, format: _Optional[_Union[CashAccountReport.Format, str]] = ..., country: _Optional[str] = ..., lang: _Optional[str] = ..., from_date: _Optional[_Union[CashAccountReport.Request.Date, _Mapping]] = ..., to_date: _Optional[_Union[CashAccountReport.Request.Date, _Mapping]] = ...) -> None: ...
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    CSV: CashAccountReport.Format
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    HTML: CashAccountReport.Format
    PDF: CashAccountReport.Format
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    XLS: CashAccountReport.Format
    content: str
    format: CashAccountReport.Format
    response_datetime: _timestamp_pb2.Timestamp
    def __init__(self, content: _Optional[str] = ..., format: _Optional[_Union[CashAccountReport.Format, str]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class CompanyProfile(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _struct_pb2.Struct
    def __init__(self, values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class CompanyRatios(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _struct_pb2.Struct
    def __init__(self, values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class Credentials(_message.Message):
    __slots__ = ["int_account", "one_time_password", "password", "totp_secret_key", "username"]
    INT_ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    ONE_TIME_PASSWORD_FIELD_NUMBER: _ClassVar[int]
    PASSWORD_FIELD_NUMBER: _ClassVar[int]
    TOTP_SECRET_KEY_FIELD_NUMBER: _ClassVar[int]
    USERNAME_FIELD_NUMBER: _ClassVar[int]
    int_account: int
    one_time_password: int
    password: str
    totp_secret_key: str
    username: str
    def __init__(self, int_account: _Optional[int] = ..., username: _Optional[str] = ..., password: _Optional[str] = ..., totp_secret_key: _Optional[str] = ..., one_time_password: _Optional[int] = ...) -> None: ...

class EstimatesSummaries(_message.Message):
    __slots__ = ["annual", "currency", "interim", "lastRetrieved", "lastUpdated", "preferredMeasure", "ric"]
    ANNUAL_FIELD_NUMBER: _ClassVar[int]
    CURRENCY_FIELD_NUMBER: _ClassVar[int]
    INTERIM_FIELD_NUMBER: _ClassVar[int]
    LASTRETRIEVED_FIELD_NUMBER: _ClassVar[int]
    LASTUPDATED_FIELD_NUMBER: _ClassVar[int]
    PREFERREDMEASURE_FIELD_NUMBER: _ClassVar[int]
    RIC_FIELD_NUMBER: _ClassVar[int]
    annual: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    currency: str
    interim: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    lastRetrieved: str
    lastUpdated: str
    preferredMeasure: str
    ric: str
    def __init__(self, annual: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., currency: _Optional[str] = ..., interim: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., lastRetrieved: _Optional[str] = ..., lastUpdated: _Optional[str] = ..., preferredMeasure: _Optional[str] = ..., ric: _Optional[str] = ...) -> None: ...

class Favourites(_message.Message):
    __slots__ = ["response_datetime", "values"]
    class List(_message.Message):
        __slots__ = ["id", "is_default", "name", "product_ids"]
        ID_FIELD_NUMBER: _ClassVar[int]
        IS_DEFAULT_FIELD_NUMBER: _ClassVar[int]
        NAME_FIELD_NUMBER: _ClassVar[int]
        PRODUCT_IDS_FIELD_NUMBER: _ClassVar[int]
        id: int
        is_default: bool
        name: str
        product_ids: _containers.RepeatedScalarFieldContainer[int]
        def __init__(self, id: _Optional[int] = ..., name: _Optional[str] = ..., is_default: bool = ..., product_ids: _Optional[_Iterable[int]] = ...) -> None: ...
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    response_datetime: _timestamp_pb2.Timestamp
    values: _containers.RepeatedCompositeFieldContainer[Favourites.List]
    def __init__(self, values: _Optional[_Iterable[_Union[Favourites.List, _Mapping]]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class FinancialStatements(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _struct_pb2.Struct
    def __init__(self, values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class LatestNews(_message.Message):
    __slots__ = ["items", "offset", "total"]
    class Request(_message.Message):
        __slots__ = ["languages", "limit", "offset"]
        LANGUAGES_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        languages: str
        limit: int
        offset: int
        def __init__(self, offset: _Optional[int] = ..., languages: _Optional[str] = ..., limit: _Optional[int] = ...) -> None: ...
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    offset: int
    total: int
    def __init__(self, items: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., offset: _Optional[int] = ..., total: _Optional[int] = ...) -> None: ...

class NewsByCompany(_message.Message):
    __slots__ = ["items", "offset", "total"]
    class Request(_message.Message):
        __slots__ = ["isin", "languages", "limit", "offset"]
        ISIN_FIELD_NUMBER: _ClassVar[int]
        LANGUAGES_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        isin: str
        languages: str
        limit: int
        offset: int
        def __init__(self, isin: _Optional[str] = ..., limit: _Optional[int] = ..., offset: _Optional[int] = ..., languages: _Optional[str] = ...) -> None: ...
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    offset: int
    total: int
    def __init__(self, items: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., offset: _Optional[int] = ..., total: _Optional[int] = ...) -> None: ...

class Order(_message.Message):
    __slots__ = ["action", "contract_size", "contract_type", "currency", "hour", "id", "is_deletable", "is_modifiable", "order_type", "price", "product", "product_id", "quantity", "retained_order", "sent_to_exchange", "size", "stop_price", "time_type", "total_order_value"]
    class Action(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class OrderType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class TimeType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class CheckingResponse(_message.Message):
        __slots__ = ["auto_fx_conversion_rate", "confirmation_id", "free_space_new", "response_datetime", "show_ex_ante_report_link", "transaction_auto_fx_opposite_surcharges", "transaction_auto_fx_surcharges", "transaction_fee", "transaction_fees", "transaction_opposite_fees", "transaction_taxes"]
        AUTO_FX_CONVERSION_RATE_FIELD_NUMBER: _ClassVar[int]
        CONFIRMATION_ID_FIELD_NUMBER: _ClassVar[int]
        FREE_SPACE_NEW_FIELD_NUMBER: _ClassVar[int]
        RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
        SHOW_EX_ANTE_REPORT_LINK_FIELD_NUMBER: _ClassVar[int]
        TRANSACTION_AUTO_FX_OPPOSITE_SURCHARGES_FIELD_NUMBER: _ClassVar[int]
        TRANSACTION_AUTO_FX_SURCHARGES_FIELD_NUMBER: _ClassVar[int]
        TRANSACTION_FEES_FIELD_NUMBER: _ClassVar[int]
        TRANSACTION_FEE_FIELD_NUMBER: _ClassVar[int]
        TRANSACTION_OPPOSITE_FEES_FIELD_NUMBER: _ClassVar[int]
        TRANSACTION_TAXES_FIELD_NUMBER: _ClassVar[int]
        auto_fx_conversion_rate: float
        confirmation_id: str
        free_space_new: float
        response_datetime: _timestamp_pb2.Timestamp
        show_ex_ante_report_link: bool
        transaction_auto_fx_opposite_surcharges: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        transaction_auto_fx_surcharges: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        transaction_fee: float
        transaction_fees: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        transaction_opposite_fees: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        transaction_taxes: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        def __init__(self, confirmation_id: _Optional[str] = ..., free_space_new: _Optional[float] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., transaction_fees: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., transaction_opposite_fees: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., transaction_taxes: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., transaction_auto_fx_surcharges: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., transaction_auto_fx_opposite_surcharges: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., auto_fx_conversion_rate: _Optional[float] = ..., transaction_fee: _Optional[float] = ..., show_ex_ante_report_link: bool = ...) -> None: ...
    class ConfirmationResponse(_message.Message):
        __slots__ = ["order_id", "response_datetime"]
        ORDER_ID_FIELD_NUMBER: _ClassVar[int]
        RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
        order_id: str
        response_datetime: _timestamp_pb2.Timestamp
        def __init__(self, order_id: _Optional[str] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    ACTION_FIELD_NUMBER: _ClassVar[int]
    BUY: Order.Action
    CONTRACT_SIZE_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_TYPE_FIELD_NUMBER: _ClassVar[int]
    CURRENCY_FIELD_NUMBER: _ClassVar[int]
    GOOD_TILL_CANCELED: Order.TimeType
    GOOD_TILL_DAY: Order.TimeType
    HOUR_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    IS_DELETABLE_FIELD_NUMBER: _ClassVar[int]
    IS_MODIFIABLE_FIELD_NUMBER: _ClassVar[int]
    LIMIT: Order.OrderType
    MARKET: Order.OrderType
    ORDER_TYPE_FIELD_NUMBER: _ClassVar[int]
    PRICE_FIELD_NUMBER: _ClassVar[int]
    PRODUCT_FIELD_NUMBER: _ClassVar[int]
    PRODUCT_ID_FIELD_NUMBER: _ClassVar[int]
    QUANTITY_FIELD_NUMBER: _ClassVar[int]
    RETAINED_ORDER_FIELD_NUMBER: _ClassVar[int]
    SELL: Order.Action
    SENT_TO_EXCHANGE_FIELD_NUMBER: _ClassVar[int]
    SIZE_FIELD_NUMBER: _ClassVar[int]
    STOP_LIMIT: Order.OrderType
    STOP_LOSS: Order.OrderType
    STOP_PRICE_FIELD_NUMBER: _ClassVar[int]
    TIME_TYPE_FIELD_NUMBER: _ClassVar[int]
    TOTAL_ORDER_VALUE_FIELD_NUMBER: _ClassVar[int]
    UNKNOWN_0: Order.TimeType
    UNKNOWN_2: Order.TimeType
    action: Order.Action
    contract_size: float
    contract_type: int
    currency: str
    hour: str
    id: str
    is_deletable: bool
    is_modifiable: bool
    order_type: Order.OrderType
    price: float
    product: str
    product_id: int
    quantity: float
    retained_order: bool
    sent_to_exchange: bool
    size: float
    stop_price: float
    time_type: Order.TimeType
    total_order_value: float
    def __init__(self, action: _Optional[_Union[Order.Action, str]] = ..., id: _Optional[str] = ..., order_type: _Optional[_Union[Order.OrderType, str]] = ..., price: _Optional[float] = ..., stop_price: _Optional[float] = ..., product_id: _Optional[int] = ..., size: _Optional[float] = ..., time_type: _Optional[_Union[Order.TimeType, str]] = ..., contract_size: _Optional[float] = ..., contract_type: _Optional[int] = ..., currency: _Optional[str] = ..., hour: _Optional[str] = ..., is_deletable: bool = ..., is_modifiable: bool = ..., product: _Optional[str] = ..., quantity: _Optional[float] = ..., total_order_value: _Optional[float] = ..., retained_order: bool = ..., sent_to_exchange: bool = ...) -> None: ...

class OrdersHistory(_message.Message):
    __slots__ = ["response_datetime", "values"]
    class Request(_message.Message):
        __slots__ = ["from_date", "to_date"]
        class Date(_message.Message):
            __slots__ = ["day", "month", "year"]
            DAY_FIELD_NUMBER: _ClassVar[int]
            MONTH_FIELD_NUMBER: _ClassVar[int]
            YEAR_FIELD_NUMBER: _ClassVar[int]
            day: int
            month: int
            year: int
            def __init__(self, year: _Optional[int] = ..., month: _Optional[int] = ..., day: _Optional[int] = ...) -> None: ...
        FROM_DATE_FIELD_NUMBER: _ClassVar[int]
        TO_DATE_FIELD_NUMBER: _ClassVar[int]
        from_date: OrdersHistory.Request.Date
        to_date: OrdersHistory.Request.Date
        def __init__(self, from_date: _Optional[_Union[OrdersHistory.Request.Date, _Mapping]] = ..., to_date: _Optional[_Union[OrdersHistory.Request.Date, _Mapping]] = ...) -> None: ...
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    response_datetime: _timestamp_pb2.Timestamp
    values: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    def __init__(self, values: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class PositionReport(_message.Message):
    __slots__ = ["content", "format", "response_datetime"]
    class Format(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class Request(_message.Message):
        __slots__ = ["country", "format", "lang", "to_date"]
        class Date(_message.Message):
            __slots__ = ["day", "month", "year"]
            DAY_FIELD_NUMBER: _ClassVar[int]
            MONTH_FIELD_NUMBER: _ClassVar[int]
            YEAR_FIELD_NUMBER: _ClassVar[int]
            day: int
            month: int
            year: int
            def __init__(self, year: _Optional[int] = ..., month: _Optional[int] = ..., day: _Optional[int] = ...) -> None: ...
        COUNTRY_FIELD_NUMBER: _ClassVar[int]
        FORMAT_FIELD_NUMBER: _ClassVar[int]
        LANG_FIELD_NUMBER: _ClassVar[int]
        TO_DATE_FIELD_NUMBER: _ClassVar[int]
        country: str
        format: PositionReport.Format
        lang: str
        to_date: PositionReport.Request.Date
        def __init__(self, format: _Optional[_Union[PositionReport.Format, str]] = ..., country: _Optional[str] = ..., lang: _Optional[str] = ..., to_date: _Optional[_Union[PositionReport.Request.Date, _Mapping]] = ...) -> None: ...
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    CSV: PositionReport.Format
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    HTML: PositionReport.Format
    PDF: PositionReport.Format
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    XLS: PositionReport.Format
    content: str
    format: PositionReport.Format
    response_datetime: _timestamp_pb2.Timestamp
    def __init__(self, content: _Optional[str] = ..., format: _Optional[_Union[PositionReport.Format, str]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class ProductSearch(_message.Message):
    __slots__ = ["offset", "products", "response_datetime", "total"]
    class Config(_message.Message):
        __slots__ = ["values"]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        values: _struct_pb2.Struct
        def __init__(self, values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...
    class RequestBonds(_message.Message):
        __slots__ = ["bond_exchange_id", "bond_issuer_type_id", "limit", "offset", "require_total", "search_text", "sort_columns", "sort_types"]
        BOND_EXCHANGE_ID_FIELD_NUMBER: _ClassVar[int]
        BOND_ISSUER_TYPE_ID_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        bond_exchange_id: int
        bond_issuer_type_id: int
        limit: int
        offset: int
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        def __init__(self, bond_issuer_type_id: _Optional[int] = ..., bond_exchange_id: _Optional[int] = ..., search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestETFs(_message.Message):
        __slots__ = ["input_aggregate_types", "input_aggregate_values", "limit", "offset", "popular_only", "require_total", "search_text", "sort_columns", "sort_types"]
        INPUT_AGGREGATE_TYPES_FIELD_NUMBER: _ClassVar[int]
        INPUT_AGGREGATE_VALUES_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        POPULAR_ONLY_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        input_aggregate_types: str
        input_aggregate_values: str
        limit: int
        offset: int
        popular_only: bool
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        def __init__(self, popular_only: bool = ..., input_aggregate_types: _Optional[str] = ..., input_aggregate_values: _Optional[str] = ..., search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestFunds(_message.Message):
        __slots__ = ["limit", "offset", "require_total", "search_text", "sort_columns", "sort_types"]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        limit: int
        offset: int
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        def __init__(self, search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestFutures(_message.Message):
        __slots__ = ["future_exchange_id", "limit", "offset", "require_total", "search_text", "sort_columns", "sort_types", "underlying_isin"]
        FUTURE_EXCHANGE_ID_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        UNDERLYING_ISIN_FIELD_NUMBER: _ClassVar[int]
        future_exchange_id: int
        limit: int
        offset: int
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        underlying_isin: str
        def __init__(self, future_exchange_id: _Optional[int] = ..., underlying_isin: _Optional[str] = ..., search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestLeverageds(_message.Message):
        __slots__ = ["input_aggregate_types", "input_aggregate_values", "limit", "offset", "popular_only", "require_total", "search_text", "sort_columns", "sort_types"]
        INPUT_AGGREGATE_TYPES_FIELD_NUMBER: _ClassVar[int]
        INPUT_AGGREGATE_VALUES_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        POPULAR_ONLY_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        input_aggregate_types: str
        input_aggregate_values: str
        limit: int
        offset: int
        popular_only: bool
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        def __init__(self, popular_only: bool = ..., input_aggregate_types: _Optional[str] = ..., input_aggregate_values: _Optional[str] = ..., search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestLookup(_message.Message):
        __slots__ = ["limit", "offset", "product_type_id", "search_text"]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        PRODUCT_TYPE_ID_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        limit: int
        offset: int
        product_type_id: int
        search_text: str
        def __init__(self, search_text: _Optional[str] = ..., limit: _Optional[int] = ..., offset: _Optional[int] = ..., product_type_id: _Optional[int] = ...) -> None: ...
    class RequestOptions(_message.Message):
        __slots__ = ["input_aggregate_types", "input_aggregate_values", "limit", "offset", "option_exchange_id", "require_total", "search_text", "sort_columns", "sort_types", "underlying_isin"]
        INPUT_AGGREGATE_TYPES_FIELD_NUMBER: _ClassVar[int]
        INPUT_AGGREGATE_VALUES_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        OPTION_EXCHANGE_ID_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        UNDERLYING_ISIN_FIELD_NUMBER: _ClassVar[int]
        input_aggregate_types: str
        input_aggregate_values: str
        limit: int
        offset: int
        option_exchange_id: int
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        underlying_isin: str
        def __init__(self, input_aggregate_types: _Optional[str] = ..., input_aggregate_values: _Optional[str] = ..., option_exchange_id: _Optional[int] = ..., underlying_isin: _Optional[str] = ..., search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestStocks(_message.Message):
        __slots__ = ["exchange_id", "index_id", "is_in_us_green_list", "limit", "offset", "require_total", "search_text", "sort_columns", "sort_types", "stock_country_id"]
        EXCHANGE_ID_FIELD_NUMBER: _ClassVar[int]
        INDEX_ID_FIELD_NUMBER: _ClassVar[int]
        IS_IN_US_GREEN_LIST_FIELD_NUMBER: _ClassVar[int]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        STOCK_COUNTRY_ID_FIELD_NUMBER: _ClassVar[int]
        exchange_id: int
        index_id: int
        is_in_us_green_list: bool
        limit: int
        offset: int
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        stock_country_id: int
        def __init__(self, is_in_us_green_list: bool = ..., index_id: _Optional[int] = ..., exchange_id: _Optional[int] = ..., stock_country_id: _Optional[int] = ..., search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    class RequestWarrants(_message.Message):
        __slots__ = ["limit", "offset", "require_total", "search_text", "sort_columns", "sort_types"]
        LIMIT_FIELD_NUMBER: _ClassVar[int]
        OFFSET_FIELD_NUMBER: _ClassVar[int]
        REQUIRE_TOTAL_FIELD_NUMBER: _ClassVar[int]
        SEARCH_TEXT_FIELD_NUMBER: _ClassVar[int]
        SORT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
        SORT_TYPES_FIELD_NUMBER: _ClassVar[int]
        limit: int
        offset: int
        require_total: bool
        search_text: str
        sort_columns: str
        sort_types: str
        def __init__(self, search_text: _Optional[str] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., require_total: bool = ..., sort_columns: _Optional[str] = ..., sort_types: _Optional[str] = ...) -> None: ...
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    PRODUCTS_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    offset: int
    products: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    response_datetime: _timestamp_pb2.Timestamp
    total: int
    def __init__(self, offset: _Optional[int] = ..., products: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., total: _Optional[int] = ...) -> None: ...

class ProductsInfo(_message.Message):
    __slots__ = ["values"]
    class Request(_message.Message):
        __slots__ = ["products"]
        PRODUCTS_FIELD_NUMBER: _ClassVar[int]
        products: _containers.RepeatedScalarFieldContainer[int]
        def __init__(self, products: _Optional[_Iterable[int]] = ...) -> None: ...
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _struct_pb2.Struct
    def __init__(self, values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class TopNewsPreview(_message.Message):
    __slots__ = ["items", "offset", "total"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    offset: int
    total: int
    def __init__(self, items: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., offset: _Optional[int] = ..., total: _Optional[int] = ...) -> None: ...

class TransactionsHistory(_message.Message):
    __slots__ = ["response_datetime", "values"]
    class Request(_message.Message):
        __slots__ = ["from_date", "group_transactions_by_order", "to_date"]
        class Date(_message.Message):
            __slots__ = ["day", "month", "year"]
            DAY_FIELD_NUMBER: _ClassVar[int]
            MONTH_FIELD_NUMBER: _ClassVar[int]
            YEAR_FIELD_NUMBER: _ClassVar[int]
            day: int
            month: int
            year: int
            def __init__(self, year: _Optional[int] = ..., month: _Optional[int] = ..., day: _Optional[int] = ...) -> None: ...
        FROM_DATE_FIELD_NUMBER: _ClassVar[int]
        GROUP_TRANSACTIONS_BY_ORDER_FIELD_NUMBER: _ClassVar[int]
        TO_DATE_FIELD_NUMBER: _ClassVar[int]
        from_date: TransactionsHistory.Request.Date
        group_transactions_by_order: bool
        to_date: TransactionsHistory.Request.Date
        def __init__(self, from_date: _Optional[_Union[TransactionsHistory.Request.Date, _Mapping]] = ..., to_date: _Optional[_Union[TransactionsHistory.Request.Date, _Mapping]] = ..., group_transactions_by_order: bool = ...) -> None: ...
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    response_datetime: _timestamp_pb2.Timestamp
    values: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    def __init__(self, values: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ..., response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class Update(_message.Message):
    __slots__ = ["alerts", "cash_funds", "historical_orders", "orders", "portfolio", "response_datetime", "total_portfolio", "transactions"]
    class Option(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class Alerts(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _struct_pb2.Struct
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...
    class CashFunds(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _struct_pb2.Struct
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...
    class HistoricalOrders(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _containers.RepeatedCompositeFieldContainer[Order]
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Iterable[_Union[Order, _Mapping]]] = ...) -> None: ...
    class Orders(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _containers.RepeatedCompositeFieldContainer[Order]
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Iterable[_Union[Order, _Mapping]]] = ...) -> None: ...
    class Portfolio(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ...) -> None: ...
    class Request(_message.Message):
        __slots__ = ["last_updated", "option"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        OPTION_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        option: Update.Option
        def __init__(self, option: _Optional[_Union[Update.Option, str]] = ..., last_updated: _Optional[int] = ...) -> None: ...
    class RequestList(_message.Message):
        __slots__ = ["values"]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        values: _containers.RepeatedCompositeFieldContainer[Update.Request]
        def __init__(self, values: _Optional[_Iterable[_Union[Update.Request, _Mapping]]] = ...) -> None: ...
    class TotalPortfolio(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _struct_pb2.Struct
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...
    class Transactions(_message.Message):
        __slots__ = ["last_updated", "values"]
        LAST_UPDATED_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        last_updated: int
        values: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
        def __init__(self, last_updated: _Optional[int] = ..., values: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ...) -> None: ...
    ALERTS: Update.Option
    ALERTS_FIELD_NUMBER: _ClassVar[int]
    CASHFUNDS: Update.Option
    CASH_FUNDS_FIELD_NUMBER: _ClassVar[int]
    HISTORICALORDERS: Update.Option
    HISTORICAL_ORDERS_FIELD_NUMBER: _ClassVar[int]
    ORDERS: Update.Option
    ORDERS_FIELD_NUMBER: _ClassVar[int]
    PORTFOLIO: Update.Option
    PORTFOLIO_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_DATETIME_FIELD_NUMBER: _ClassVar[int]
    TOTALPORTFOLIO: Update.Option
    TOTAL_PORTFOLIO_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS: Update.Option
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    alerts: Update.Alerts
    cash_funds: Update.CashFunds
    historical_orders: Update.HistoricalOrders
    orders: Update.Orders
    portfolio: Update.Portfolio
    response_datetime: _timestamp_pb2.Timestamp
    total_portfolio: Update.TotalPortfolio
    transactions: Update.Transactions
    def __init__(self, response_datetime: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., alerts: _Optional[_Union[Update.Alerts, _Mapping]] = ..., cash_funds: _Optional[_Union[Update.CashFunds, _Mapping]] = ..., historical_orders: _Optional[_Union[Update.HistoricalOrders, _Mapping]] = ..., orders: _Optional[_Union[Update.Orders, _Mapping]] = ..., portfolio: _Optional[_Union[Update.Portfolio, _Mapping]] = ..., total_portfolio: _Optional[_Union[Update.TotalPortfolio, _Mapping]] = ..., transactions: _Optional[_Union[Update.Transactions, _Mapping]] = ...) -> None: ...



import logging
from datetime import datetime, timedelta
from typing import Any

import json
import polars as pl
import requests
from isodate import parse_duration

from degiro_connector.core.constants import urls
from degiro_connector.quotecast.models.chart import Chart, ChartRequest, Series
from degiro_connector.core.models.model_connection import ModelConnection
from degiro_connector.core.models.model_session import ModelSession


class SeriesFormatter:
    @staticmethod
    def is_timeseries(series: Series) -> bool:
        return series.type in ["time", "ohlc"]

    @staticmethod
    def parse_date_and_resolution(times: str) -> tuple[datetime, timedelta]:
        """Extract the interval of a timeserie.
        Args:
            times (str):
                Combination of `start date` and `resolution` of the serie.
                Example :
                    times = "2021-10-28/P6M"
                    times = "2021-11-03T00:00:00/PT1H"
        Raises:
            AttributeError:
                if the resolution is unknown.
        Returns:
            tuple[datetime, timedelta]:
                Start datetime and resolution.
        """
        start, resolution = times.split(sep="/", maxsplit=1)
        start_datetime = datetime.fromisoformat(start)
        resolution_interval = parse_duration(resolution)

        return start_datetime, resolution_interval

    @staticmethod
    def format_timestamp(
        df: pl.DataFrame,
        column: str,
        start: datetime,
        resolution: timedelta,
    ):
        df = df.with_columns((pl.col(column) * resolution).cast(pl.Duration) + start)

        return df

    @classmethod
    def format_timeseries(
        cls,
        series: Series,
        columns: list[str] | None = None,
    ) -> pl.DataFrame:
        if series.type is None:
            raise TypeError("Can't parse `None` series.")

        if not cls.is_timeseries(series=series):
            raise TypeError(f"Only timeseries can be formatted, type={series.type}")

        if series.times is None or series.type not in ["time", "ohlc"]:
            raise AttributeError("The attributes `times` is empty.")

        if columns:
            pass
        elif series.id.startswith("price"):
            columns = ["timestamp", "price"]
        elif series.id.startswith("volume"):
            columns = ["timestamp", "volume"]
        elif series.id.startswith("ohlc"):
            columns = ["timestamp", "open", "high", "low", "close"]
        else:
            columns = None

        df = pl.DataFrame(
            data=series.data,
            orient="row",
            schema=columns,
        )
        start, resolution = cls.parse_date_and_resolution(times=series.times)
        column = df.columns[0]
        formatted_df = cls.format_timestamp(
            df=df, column=column, start=start, resolution=resolution
        )

        return formatted_df

    @classmethod
    def format_series(
        cls,
        series: Series,
        columns: list[str] | None = None,
    ) -> pl.DataFrame:
        if cls.is_timeseries(series=series):
            df = cls.format_timeseries(
                series=series,
                columns=columns,
            )
        else:
            df = pl.DataFrame(series.data)

        return df


class ChartFetcher:
    @staticmethod
    def build_logger() -> logging.Logger:
        return logging.getLogger(__name__)

    @staticmethod
    def build_session(headers: dict[str, str] | None = None) -> requests.Session:
        return ModelSession.build_session(headers=headers)

    @property
    def user_token(self):
        return self.__user_token

    @property
    def connection_storage(self):
        return self.__connection_storage

    @property
    def logger(self):
        return self.__logger

    @property
    def session_storage(self):
        return self._session_storage

    @staticmethod
    def build_params(
        chart_request: ChartRequest,
        user_token: int,
    ) -> dict[str, Any]:
        chart_request.user_token = chart_request.user_token or user_token
        params = chart_request.model_dump(
            by_alias=True,
            exclude={"override"},
            exclude_none=True,
            mode="json",
        )
        params.update(chart_request.override)

        return params

    def get_chart(
        self,
        chart_request: ChartRequest,
        logger: logging.Logger | None = None,
        raw: bool = False,
        session: requests.Session | None = None,
    ) -> Chart | None:
        """Fetches chart's data.
        Args:
            request (ChartRequest):
                Example :
                    chart_request = ChartRequest(
                        culture = "fr-FR",
                        # override={
                        #     "resolution": "P1D",
                        #     "period": "P1W",
                        # },
                        period = Interval.P1D,
                        requestid = "1",
                        resolution = Interval.PT1H,
                        series=[
                            "issueid:360148977",
                            # "price:issueid:360148977",
                            # "ohlc:issueid:360148977",
                            # "volume:issueid:360148977",
                            # "vwdkey:AAPL.BATS,E",
                            # "price:vwdkey:AAPL.BATS,E",
                            # "ohlc:vwdkey:AAPL.BATS,E",
                            # "volume:vwdkey:AAPL.BATS,E",
                        ],
                        tz = "Europe/Paris",
                    )

                    The parameter `request.override` allows overriding
                    the request sent to Degiro's API.
            user_token (int):
                User identifier in Degiro's API.
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            Chart: Data of the chart.
        """

        session = self.session_storage.session
        logger = self.logger
        user_token = self.__user_token

        if logger is None:
            logger = self.build_logger()
        if session is None:
            session = self.build_session()

        url = urls.CHART
        params = self.build_params(
            chart_request=chart_request,
            user_token=user_token,
        )

        http_request = requests.Request(method="GET", url=url, params=params)
        prepped = session.prepare_request(http_request)

        try:
            response = session.send(prepped)
            response.raise_for_status()
            response_map = json.loads(
                response.text[len(chart_request.callback) + 1 : -1]
            )

            if raw is True:
                chart = response_map
            else:
                chart = Chart.model_validate(obj=response_map)

            return chart
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def __init__(
        self,
        user_token: int,
        connection_storage: ModelConnection | None = None,
        logger: logging.Logger | None = None,
        session_storage: ModelSession | None = None,
    ):
        self.__user_token = user_token
        self.__connection_storage = connection_storage or ModelConnection(timeout=600)
        self.__logger = logger or logging.getLogger(self.__module__)
        self._session_storage = session_storage or ModelSession(
            hooks=self.__connection_storage.build_hooks(),
        )
